from django.conf import settings
from django.db import models
from django.utils.encoding import force_str
from django.utils.timezone import now as tznow

from nano.tools.models import GenericForeignKeyAbstractModel
from nano.tools.models import UnorderedTreeManager
from nano.tools.models import UnorderedTreeMixin
from nano.tools.models import get_manager


COMMENT_MAX_LENGTH = getattr(settings,'COMMENT_MAX_LENGTH',3000)


class Comment(GenericForeignKeyAbstractModel, UnorderedTreeMixin):
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        blank=True,
        null=True,
        related_name="%(class)s_comments",
    )
    comment = models.TextField(max_length=COMMENT_MAX_LENGTH)
    comment_xhtml = models.TextField(editable=False)
    added = models.DateTimeField(default=tznow)
    is_visible = models.BooleanField(default=True)
    is_scrambled = models.BooleanField(default=False)

    objects = models.Manager()
    tree = UnorderedTreeManager()

    class Meta:
        db_table = "nano_comments_comment"
        ordering = ('added',)
        get_latest_by = 'added'

    def __str__(self):
        comment = self.comment
        if len(comment) > 50:
            comment = self.comment[:49]+'...'
        return f"{self.user}: {comment}"

    def get_content_object_url(self):
        return self.content_object.get_absolute_url() or ''

    def get_absolute_url(self, anchor_pattern="#c%(id)s"):
        content_url = self.get_content_object_url()
        anchor = anchor_pattern % self.__dict__
        if content_url:
            return content_url + anchor
        return anchor

    def roots(self):
        manager = get_manager(self)
        tree = manager.filter(part_of__isnull=True)
        return tree.filter(content_type=self.content_type, object_pk=self.object_pk)

    def get_path(self):
        manager = get_manager(self)
        return [manager.get(id=p).filter(content_type=self.content_type, object_pk=self.object_pk)
                for p in force_str(self.path).split(self._sep) if p]

    def descendants(self):
        manager = get_manager(self)
        tree = manager.filter(path__startswith=self.path).exclude(id=self.id)
        return tree.filter(content_type=self.content_type, object_pk=self.object_pk)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2018-10-20 13:40
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Mark',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_pk', models.TextField(verbose_name='object ID')),
                ('marked_at', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date/time marked')),
                ('comment', models.CharField(blank=True, max_length=256, null=True)),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='content_type_set_for_mark', to='contenttypes.ContentType', verbose_name='content type')),
                ('marked_by', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='marks', to=settings.AUTH_USER_MODEL, verbose_name='user')),
            ],
            options={
                'ordering': ('marked_at',),
                'db_table': 'nano_mark',
                'get_latest_by': 'marked_at',
            },
        ),
        migrations.CreateModel(
            name='MarkType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=32)),
                ('slug', models.SlugField(unique=True)),
                ('hide', models.BooleanField(default=False)),
                ('verify', models.BooleanField(default=False)),
                ('permanent', models.BooleanField(default=False)),
            ],
            options={
                'db_table': 'nano_mark_marktype',
            },
        ),
        migrations.AddField(
            model_name='mark',
            name='marktype',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='mark.MarkType'),
        ),
    ]



from django.conf import settings

NANO_BLOG_TAGS = None

# Optional support for django-taggit
try:
    if ('taggit' in settings.INSTALLED_APPS 
        and getattr(settings, 'NANO_BLOG_USE_TAGS', False)):
        import taggit as NANO_BLOG_TAGS
except ImportError:
    pass

NANO_BLOG_SPECIAL_TAGS = getattr(settings, 'NANO_BLOG_SPECIAL_TAGS', ('pinned',))



from django.test import TestCase
from django.contrib.auth import get_user_model

from nano.mark.models import Mark, MarkType
from nano.mark.tests.models import Item


class MarkTypeTest(TestCase):

    def setUp(self):
        self.fave = MarkType.objects.create(name='Fave')
        self.flag = MarkType.objects.create(name='Flag')
        self.scrambled = MarkType.objects.create(name='Scrambled')
        self.removed = MarkType.objects.create(name='Removed')

    def test_str(self):
        self.assertEqual(str(self.fave), 'Fave')


class MarkTest(TestCase):

    def setUp(self):
        User = get_user_model()
        self.fave = MarkType.objects.create(name='Fave')
        self.flag = MarkType.objects.create(name='Flag')
        self.scrambled = MarkType.objects.create(name='Scrambled')
        self.removed = MarkType.objects.create(name='Removed')
        self.item = Item.objects.create(slug='test')
        self.user = User.objects.create(username='user1')

    def test_str(self):
        item = Mark(marktype=self.fave, marked_by=self.user, object_pk=str(self.item.pk))
        self.assertEqual(str(item), "%s have marked %s" % (self.user, item.content_object))


class MarkedMixinTest(TestCase):
    def setUp(self):
        User = get_user_model()
        self.fave = MarkType.objects.create(name='Fave')
        self.flag = MarkType.objects.create(name='Flag')
        self.scrambled = MarkType.objects.create(name='Scrambled')
        self.removed = MarkType.objects.create(name='Removed')
        self.item = Item.objects.create(slug='test')
        self.user = User.objects.create(username='user1')
 
    def test_faved(self):
        self.item 



# coding: utf-8

from django.utils.timezone import now as tznow
from django.test import TestCase

from nano.tools.models import UnorderedTreeManager
from nano.tools.models import UnorderedTreeMixin


class TreeItem(UnorderedTreeMixin):

    tree = UnorderedTreeManager()



from django.urls import include, path, re_path

from nano.blog import views


urlpatterns = [
    re_path(r'^(?P<year>\d{4})/', include([
        re_path(r'^(?P<month>[01]\d)/(?P<day>[0123]\d)/$', views.list_entries_by_date),
        re_path(r'^(?P<month>[01]\d)/$', views.list_entries_by_year_and_month),
        path('', views.list_entries_by_year),
    ])),
    path('latest/',                views.list_latest_entries),
    path('today/',                 views.list_entries_for_today),
    path('',                       views.list_entries),
]



from unittest import TestCase as LightTestCase
from datetime import timedelta

from django.test import TestCase
from django.contrib.auth import get_user_model
from django.utils.timezone import now as tznow

from nano.activation.models import activate, Key, ActivationKeyError
from nano.activation import to_base, NUMERALS, generate_keys, baseNgenerator

class KeyTest(TestCase):

    def setUp(self):
        User = get_user_model()
        self.user = User.objects.create(username='test')

    def test_str(self):
        item = Key.objects.create(key='test')
        pub_date = item.pub_date
        expected = "test (None) %s %s" % (pub_date, '')
        self.assertEqual(str(item), expected)

    def test_activate_method(self):
        item = Key.objects.create(key='test')
        item.activate(self.user)
        self.assertIsNotNone(item.activated)

    def test_activate_method_already_activated(self):
        item = Key.objects.create(key='test')
        item.activate(self.user)
        with self.assertRaises(ActivationKeyError) as error:
            item.activate(self.user)
            self.assertEqual(error, 'Key has already been activated')

    def test_activate_function(self):
        with self.assertRaises(ActivationKeyError) as error:
            activate('foo', self.user)
            self.assertEqual(error, 'Key foo does not exist, typo?')
        item = Key.objects.create(key='bar')
        item = activate(item.key, self.user)
        self.assertIsNotNone(item.activated)

    def test_activate_expired_key(self):
        now = tznow()
        item = Key(key='test', expires=now)
        with self.assertRaises(ActivationKeyError) as error:
            item.activate(self.user)
            self.assertEqual(error, 'Key expired on %s' % item.expires)

class KeyManagerTest(TestCase):

    def setUp(self):
        User = get_user_model()
        self.user = User.objects.create(username='test')
        self.keys = []
        for i in range(10):
            key = Key.objects.create(key=str(i))
            self.keys.append(key)

    def test_get_all_keys(self):
        result1 = Key.objects.order_by('id')
        self.assertEqual(self.keys, list(result1))
        result2 = Key.objects.available().order_by('id')
        self.assertEqual(self.keys, list(result2))
        self.assertEqual(list(result1), list(result2))

    def test_get_expired_keys(self):
        now = tznow() - timedelta(days=1)
        for key in self.keys[5:]:
            key.expires = now
            key.save()
        self.assertEqual(
            list(Key.objects.expired().order_by('id')), 
            self.keys[5:]
        )
        self.assertEqual(
            list(Key.objects.available().order_by('id')), 
            self.keys[:5]
        )

    def test_get_activated_keys(self):
        now = tznow()
        for key in self.keys[5:]:
            key.activated = now
            key.save()
        self.assertEqual(
            list(Key.objects.activated().order_by('id')), 
            self.keys[5:]
        )
        self.assertEqual(
            list(Key.objects.available().order_by('id')), 
            self.keys[:5]
        )

    def test_activate_keys(self):
        now = tznow() - timedelta(days=1)
        for i in range(5):
            Key.objects.activate(str(i), self.user)
        self.assertEqual(
            list(Key.objects.activated().order_by('id')), 
            self.keys[:5]
        )
        self.assertEqual(
            list(Key.objects.available().order_by('id')), 
            self.keys[5:]
        )
        # Expired key
        self.keys[-1].expires = now
        self.keys[-1].save()
        with self.assertRaises(ActivationKeyError) as error:
            Key.objects.activate(self.keys[-1].key, self.user)
            self.assertEqual(error, 'Key expired on %s' % now)
        # Activated key
        with self.assertRaises(ActivationKeyError) as error:
            Key.objects.activate(self.keys[0].key, self.user)
            self.assertEqual(error, 'Key has already been activated')

class FunctionTest(LightTestCase):
    
    def test_to_base(self):
        # must succeed
        result, expected = to_base(0, 20), '0'
        self.assertEqual(result, expected)
        result, expected = to_base(7, 1), '1'*7
        self.assertEqual(result, expected)
        result, expected = to_base(7, 5), '12'
        self.assertEqual(result, expected)
        result, expected = to_base(-7, 5), '-12'
        self.assertEqual(result, expected)

        # must fail
        with self.assertRaises(ValueError) as error:
            maxbase = len(NUMERALS)
            result, expected = to_base(7, maxbase+1)
            self.assertEqual(error, "<base> must be in the range [1, %i>" % maxbase)
        with self.assertRaises(ValueError) as error:
            result, expected = to_base('q', 10)
            self.assertEqual(error, 'invalid literal for int() with base 10: \'q\'')
        with self.assertRaises(ValueError) as error:
            result, expected = to_base(1, 'q')
            self.assertEqual(error, 'invalid literal for int() with base 10: \'q\'')

    def test_generate_keys(self):
        amount = 10
        result = generate_keys(baseNgenerator(), amount=amount)
        self.assertEqual(amount, len(set(result)))



from django.shortcuts import render
from django.urls import path

from nano.activation.views import activate_key


urlpatterns = [
    path('activate',       activate_key, name='nano-activate-key'),
    path('activation_ok/', render,
                           {'template_name': 'nano/activation/activated.html'},
                           name='nano-activation-ok'),
]



from django.test import TestCase

from nano.privmsg.models import PM

from django.contrib.auth import get_user_model

class PMTest(TestCase):
    def setUp(self):
        User = get_user_model()
        self.sender = User.objects.create(username='Sender')
        self.recipient = User.objects.create(username='Recipient')

    def test_str(self):
        item = PM(subject='test', text='Test')
        self.assertEqual(str(item), 'test')
        test_content = '0123456789'*10
        item = PM(text=test_content)
        self.assertTrue(len(str(item)) <= 64)
        self.assertEqual(str(item), test_content[:64])

    def test_save(self):
        test_content = '0123456789'*10
        item = PM(text=test_content, sender=self.sender, recipient=self.recipient)
        item.save()
        self.assertEqual(item.subject, test_content[:63]+'\u2026')

    def test_is_deleted(self):
        item = PM.objects.create(text='test', sender=self.sender, recipient=self.recipient)
        self.assertFalse(item.is_deleted())
        item.sender_deleted = True
        self.assertFalse(item.is_deleted())
        item.recipient_deleted = True
        self.assertTrue(item.is_deleted())

    def test_delete(self):
        item = PM.objects.create(text='test', sender=self.sender, recipient=self.recipient)
        item.delete()
        try:
            item = PM.objects.get(text='test')
        except PM.DoesNotExist:
            self.fail('PM deleted even though not marked for deletion')
        item.sender_deleted = True
        item.recipient_deleted = True
        item.delete()
        try:
            item = PM.objects.get(text='test')
            self.fail('PM not deleted even though marked for deletion')
        except PM.DoesNotExist:
            pass



"""
Mixin-models, with minimal example implementations.

Utils relevant for models and model classes.
"""
from django.conf import settings
from django.contrib.contenttypes import fields as generic
from django.db import models
from django.db.models.base import ModelBase
from django.utils.encoding import force_str
from django.utils.translation import gettext_lazy as _


def get_manager(model):
    if not isinstance(model, ModelBase):
        model = model.__class__
    return model._default_manager


class UnorderedTreeManager(models.Manager):
    def roots(self):
        "Return a list of tree roots, nodes having no parents"
        return self.get_queryset().filter(part_of__isnull=True)


class UnorderedTreeMixin(models.Model):
    part_of = models.ForeignKey(
        'self',
        on_delete=models.CASCADE,
        blank=True,
        null=True,
        default=None,
        related_name='has_%(class)s_children'
    )
    path = models.CharField(max_length=255, blank=True, default='')

    _sep = '/'

    class Meta:
        abstract = True

    def save(self, *args, **kwargs):
        if not self.id:
            super(UnorderedTreeMixin, self).save(*args, **kwargs)

        self._set_path()
        super(UnorderedTreeMixin, self).save(*args, **kwargs)


    def _set_path(self):

        if self.part_of:
            self.path = "%s%i/" % (self.part_of.path, self.id)
        else:
            self.path = "%i/" % self.id

    @property
    def level(self):
        "Count how far down in the tree self is"
        return force_str(self.path).count(self._sep)

    @classmethod
    def roots(cls):
        "Get all roots, nodes without parents"
        return cls.tree.roots()

    @classmethod
    def get_path_for_tree(cls, treeobj):
        "Get all ancestors in a path"
        return [cls.tree.get(id=p) for p in force_str(treeobj.path).split(treeobj._sep) if p]

    def get_path(self):
        "Get all ancestors, ordered from root to self"
        return self.get_path_for_tree(self)

    @classmethod
    def get_descendants_for_tree(cls, treeobj):
        "Get all descendants in no particular order"
        return cls.tree.filter(path__startswith=treeobj.path).exclude(id=treeobj.id)

    def descendants(self):
        "Get all descendants in no particular order"
        return self.get_descendants_for_tree(self)

    def parent(self):
        "Get parent of self"
        return self.part_of

    def siblings(self):
        "Get all nodes with the same parent"
        if not self.part_of: return []
        return [p for p in self.part_of.descendants() if p.level == self.level]

    def children(self):
        "Get nodes that have self as parent"
        return [p for p in self.descendants() if p.level == self.level + 1]

    def is_sibling_of(self, node):
        "Check if <node> has the same parent as self"
        return self.part_of == node.part_of

    def is_child_of(self, node):
        "Check if <node> is the parent of self"
        return self.part_of == node

    def is_root(self):
        """Check if self is a root. Roots have no parents"""
        return not bool(self.part_of)

    def is_leaf(self):
        """Check if self is a leaf. Leaves have no descendants"""
        return self.descendants().count() == 0


class AbstractText(models.Model):
    "Denormalized storage of text"
    DEFAULT_TYPE = 'plaintext'
    text = models.TextField()
    text_formatted = models.TextField(editable=False)
    text_type = models.CharField(max_length=64, default=DEFAULT_TYPE)

    class Meta:
        abstract = True

    def save(self, formatters=None, *args, **kwargs):
        if self.text_type == self.DEFAULT_TYPE:
            self.text_formatted = self.text
        else:
            if formatters:
                self.text_formatted = formatters(self.text_type)
        super(AbstractText, self).save(*args, **kwargs)


class GenericForeignKeyAbstractModel(models.Model):
    """
    An abstract base class for models with one GenericForeignKey
    """

    # Content-object field
    content_type = models.ForeignKey(
        'contenttypes.ContentType',
        on_delete=models.CASCADE,
        verbose_name=_('content type'),
        related_name="content_type_set_for_%(class)s",
    )
    object_pk = models.TextField(_('object ID'))
    content_object = generic.GenericForeignKey(ct_field="content_type", fk_field="object_pk")

    class Meta:
        abstract = True



from math import ceil

from django import template
from django.urls import reverse
from django.utils.html import format_html, mark_safe

from nano.badge.models import Badge
from nano.tools import grouper

register = template.Library()

# \u25cf BLACK CIRCLE
# \u26ab MEDIUM BLACK CIRCLE (not as common)
SYMBOLS = {
    100: '\u25cf',
    200: '\u25cf',
    300: '\u25cf',
}

SYMBOL_NAMES = {
    100: 'bronze',
    200: 'silver',
    300: 'gold',
}

def sum_badges(user):
    levels = {}
    for badge in user.badges.all():
        levels[badge.level] = levels.setdefault(badge.level, 0) + 1

    return levels

def get_badges_for_user(user):
    inner_template = '<span class="b{}" title="{} {} badge{}">{}</span>{}'
    levels = sum_badges(user)
    sorted_levels = reversed(sorted(levels.keys()))
    badge_list = []
    for level in sorted_levels:
        name = SYMBOL_NAMES[level]
        symbol = SYMBOLS[level]
        num_levels = levels[level]
        plural = 's' if num_levels > 1 else ''
        badge_list.append(format_html(inner_template, level, num_levels, name, plural, symbol, num_levels))
    return badge_list

@register.simple_tag
def show_badges(user):
    outer_template = '<span>{}</span>'
    badge_list = get_badges_for_user(user)

    if badge_list:
        out = format_html(outer_template, mark_safe('\xa0'.join(badge_list)))
        return out
    return ''

@register.simple_tag
def show_badges_as_table(user, cols=4):
    outer_template = '<table>{}</table>'
    cell = '<td>{}</td>'
    row = '<tr>{}</tr>\n'
    single_col = '<tr><td>{}</td></tr>\n'

    badge_list = get_badges_for_user(user)

    if cols == 1:
        return [format_html(single_col, badge) for badge in badge_list]
    elif cols > 1:
        piecesize = int(ceil(len(badge_list) / float(cols)))
        badge_lists = grouper(piecesize, badge_list)
        outer = []
        go_over = list(range(cols))
        for p in range(piecesize):
            inner = []
            for i in go_over:
                inner.append(format_html(cell, badge_list[i][p]))
            outer.append(format_html(row, ''.join(inner)))

    return format_html(outer_template, ''.join(outer))

@register.simple_tag
def show_badge(badge):
    if not badge: return ''
    template = '<span class="badge"><a href="{link}"><span class="b{level}" >{symbol}</span> {name}</a></span>'
    fillin = {
        'level': badge.level,
        'symbol': SYMBOLS[badge.level],
        'name': badge.name,
        'link': mark_safe(reverse('badge-detail', args=[badge.id])),
    }
    return format_html(template, **fillin)

@register.simple_tag
def show_badge_and_freq(badge):
    template = '<span class="badge-freq">{} ({})</span>'
    badge_text = show_badge(badge)
    return format_html(template, badge_text, badge.receivers.count())



from django.test import TestCase
from django.db import models
from django.contrib.auth import get_user_model
from django.contrib.contenttypes.models import ContentType

from nano.comments.models import Comment
from nano.comments.tests.models import Item


class CommentTest(TestCase):

    def setUp(self):
        User = get_user_model()
        self.user = User.objects.create(username='user')
        self.item = Item.objects.create(slug='Commented item')
        item_dict = {
            'object_pk': self.item.pk,
            'content_type': ContentType.objects.get_for_model(Item),
        }
        self.root1 = Comment(user=self.user, comment='Root 1', **item_dict).save()
        self.child1 = Comment(user=self.user, comment='Child 1', part_of=self.root1, **item_dict).save()
        self.grandchild1 = Comment(user=self.user, comment='Grandchild 1', part_of=self.child1, **item_dict).save()
        self.child2 = Comment(user=self.user, comment='Child 2', part_of=self.root1, **item_dict).save()
        self.root2 = Comment(user=self.user, comment='Root 2', **item_dict).save()

    def test_str(self):
        item = Comment(user=self.user, comment='Test')
        self.assertEqual(str(item), '%s: Test...' % str(self.user))




# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2018-10-20 13:41
from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='QA',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('question', models.CharField(max_length=255)),
                ('answer', models.TextField()),
                ('last_modified', models.DateTimeField(default=django.utils.timezone.now, editable=False)),
            ],
            options={
                'db_table': 'nano_faq_qa',
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2018-10-20 13:37
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='PM',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.TextField()),
                ('text_formatted', models.TextField(editable=False)),
                ('text_type', models.CharField(default='plaintext', max_length=64)),
                ('subject', models.CharField(blank=True, default='', max_length=64)),
                ('sent', models.DateTimeField(default=django.utils.timezone.now, editable=False)),
                ('sender_deleted', models.BooleanField(default=False)),
                ('recipient_archived', models.BooleanField(default=False)),
                ('recipient_deleted', models.BooleanField(default=False)),
                ('recipient', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='pms_received', to=settings.AUTH_USER_MODEL)),
                ('sender', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='pms_sent', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'db_table': 'nano_privmsg_pm',
            },
        ),
    ]



from datetime import datetime

from django.test import TestCase

from nano.blog.models import Entry
from nano.blog.views import ListBlogView

class BlogMixinTest(TestCase):

    def test_get_context_data(self):
        view = ListBlogView()
        view.object_list = None
        context = view.get_context_data(object_list=None)
        self.assertEqual(context['me'], 'news')

class EntryTest(TestCase):

    def test_str(self):
        pub_date = datetime(2014, 4, 1, 0)
        e = Entry(headline='test', content='test content', pub_date=pub_date)
        e.save()
        self.assertEqual('test', str(e))



from django.contrib.contenttypes.views import shortcut
from django.urls import path, re_path

from nano.comments import views


urlpatterns = [
    re_path(r'^cr/(\d+)/(.+)/$', shortcut, name='comments-url-redirect'),
    path('post',     views.post_comment, name='comments-post-comment'),
    path('',         views.list_comments, name='comments-list-comments'),
]



from django.test import TestCase

from nano.countries.models import Country

class CountryTest(TestCase):

    def test_str(self):
        item = Country(iso='no', name='Norway')
        self.assertEqual(str(item), item.name)

    def test_printable_name(self):
        item = Country(iso='no', name='Norway')
        self.assertEqual(item.printable_name, item.name)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2018-10-20 13:41
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Chunk',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField()),
                ('content', models.TextField()),
            ],
            options={
                'db_table': 'nano_chunk_chunk',
            },
        ),
    ]



# -*- coding: utf-8 -*-

from django.test import TestCase
from django.contrib.auth import get_user_model

from nano.badge.models import Badge
from nano.badge import add_badge, batchbadge
from nano.badge.views import ListBadgeView
from nano.badge.templatetags.badge_tags import sum_badges
from nano.badge.templatetags.badge_tags import get_badges_for_user
from nano.badge.templatetags.badge_tags import show_badges
from nano.badge.templatetags.badge_tags import show_badge


class BadgeTagsSmallfuncsTest(TestCase):

    def setUp(self):
        User = get_user_model()
        self.badgeless_user = User.objects.create(username='badgeless')
        self.badge = Badge.objects.create(name='test', description='Test')
        self.users = []
        for i in range(10):
            self.users.append(User.objects.create(username='user%i' % i))
        for user in self.users[:5]:
            self.badge.receivers.add(user)

    def test_sum_badges_no_badges(self):
        result = sum_badges(self.badgeless_user)
        self.assertEqual(result, {})

    def test_sum_badges(self):
        for user in self.users[:5]:
            self.assertEqual(sum_badges(user), {100: 1})

    def test_get_badges_for_user_no_badges(self):
        result = get_badges_for_user(self.badgeless_user)
        self.assertEqual(result, [])

    def test_get_badges_for_user_no_badges(self):
        expected = ['<span class="b100" title="1 bronze badge">●</span>1']
        result = get_badges_for_user(self.users[0])
        self.assertEqual(result, expected)


class BadgeTagsShowBadges(TestCase):

    def setUp(self):
        User = get_user_model()
        self.badgeless_user = User.objects.create(username='badgeless')
        self.badge = Badge.objects.create(name='test', description='Test')
        self.users = []
        for i in range(10):
            self.users.append(User.objects.create(username='user%i' % i))
        for user in self.users[:5]:
            self.badge.receivers.add(user)

    def test_show_badges_no_badges(self):
        result = show_badges(self.badgeless_user)
        self.assertEqual(result, '')

    def test_show_badges(self):
        expected = '<span><span class="b100" title="1 bronze badge">●</span>1</span>'
        result = show_badges(self.users[0])
        self.assertEqual(result, expected)


class BadgeTagsShowBadge(TestCase):

    def setUp(self):
        User = get_user_model()
        self.badgeless_user = User.objects.create(username='badgeless')
        self.badge = Badge.objects.create(name='test', description='Test')
        self.users = []
        for i in range(10):
            self.users.append(User.objects.create(username='user%i' % i))
        for user in self.users[:5]:
            self.badge.receivers.add(user)

    def test_show_badges(self):
        expected = '<span class="badge"><a href="/badge/1/"><span class="b100" >●</span> test</a></span>'
        result = show_badge(self.badge)
        self.assertEqual(result, expected)



from django.urls import include, path, re_path

from nano.privmsg import views


urlpatterns = [
    re_path(r'^(?P<msgid>[1-9][0-9]*)/', include([
        path('archive', views.move_to_archive, name='archive_pm'),
        path('delete', views.delete, name='delete_pm'),
    ])),
    #re_path(r'^(?:(?P<action>(archive|sent))/?)?$', views.show_pms, name='show_pms'),
    path('add', views.add_pm, name='add_pm'),
    path('archive/', views.show_pm_archived, name='show_archived_pms'),
    path('sent/', views.show_pm_sent, name='show_sent_pms'),
    path('', views.show_pm_received, name='show_pms'),
    #re_path(r'^$', views.show_pms, {u'action': u'received'}, name='show_pms'),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2018-10-21 13:36
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Key',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.CharField(max_length=255)),
                ('group', models.SlugField(blank=True, max_length=32, null=True)),
                ('pub_date', models.DateTimeField(auto_now_add=True)),
                ('expires', models.DateTimeField(blank=True, null=True)),
                ('activated', models.DateTimeField(blank=True, null=True)),
                ('activated_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='activation_keys', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'get_latest_by': 'pub_date',
                'ordering': ('-pub_date',),
                'db_table': 'nano_activation_code',
            },
        ),
    ]



from django.db import models

from nano.blog.settings import NANO_BLOG_TAGS

# Optional support for django-taggit
# For some unfathomable reason this cannot be imported inside the class
# as of Django 1.3
if NANO_BLOG_TAGS:
    from taggit.managers import TaggableManager


class Entry(models.Model):
    headline = models.CharField(max_length=255)
    content = models.TextField()
    pub_date = models.DateTimeField()

    # Optional support for django-taggit
    if NANO_BLOG_TAGS:
        tags = TaggableManager(blank=True)

    class Meta:
        db_table = 'nano_blog_entry'
        verbose_name_plural = 'entries'
        ordering = ('-pub_date',)
        get_latest_by = 'pub_date'

    def __str__(self):
        return self.headline



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2018-10-21 13:38
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Country',
            fields=[
                ('iso', models.CharField(max_length=2, primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=128, verbose_name='Country name')),
            ],
            options={
                'verbose_name_plural': 'countries',
                'db_table': 'nano_country',
                'verbose_name': 'country',
            },
        ),
    ]



from django.test import TestCase
from django.template.engine import Engine
from django.template import TemplateDoesNotExist

from nano.chunk.models import Chunk


class ChunkTest(TestCase):

    def setUp(self):
        engine = Engine(loaders=['nano.chunk.loaders.ChunkLoader'])
        self.loader = engine.template_loaders[0]

    def test_str(self):
        item = Chunk(slug='test', content='Test')
        self.assertEqual(str(item), item.slug)

    def test_non_existing(self):
        with self.assertRaises(TemplateDoesNotExist):
            self.loader.load_template_source("not-existing.html")

    def test_existing(self):
        content = 'Test!'
        template_name = 'test'
        chunk = Chunk.objects.create(slug=template_name, content=content)
        result = self.loader.load_template_source("test")
        self.assertEqual(result[0], content)
        self.assertEqual(result[1], template_name)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2018-10-20 13:48
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Comment',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('path', models.CharField(blank=True, default='', max_length=255)),
                ('object_pk', models.TextField(verbose_name='object ID')),
                ('comment', models.TextField(max_length=3000)),
                ('comment_xhtml', models.TextField(editable=False)),
                ('added', models.DateTimeField(default=django.utils.timezone.now)),
                ('is_visible', models.BooleanField(default=True)),
                ('is_scrambled', models.BooleanField(default=False)),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='content_type_set_for_comment', to='contenttypes.ContentType', verbose_name='content type')),
                ('part_of', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='has_comment_children', to='comments.Comment')),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='comment_comments', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'get_latest_by': 'added',
                'db_table': 'nano_comments_comment',
                'ordering': ('added',),
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2018-10-20 13:53
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Badge',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('level', models.PositiveIntegerField(default=100)),
                ('name', models.CharField(max_length=20, unique=True)),
                ('description', models.TextField()),
                ('receivers', models.ManyToManyField(blank=True, related_name='badges', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'db_table': 'nano_badge_badge',
            },
        ),
    ]



from django.utils.timezone import now as tznow
from django.test import TestCase

from nano.faq.models import QA

class QATest(TestCase):

    def test_str(self):
        item = QA(question='blbl', answer='fofo')
        self.assertEqual(str(item), item.question)

    def test_save(self):
        item = QA(question='blbl', answer='fofo')
        item.save()
        self.assertNotEqual(item.last_modified, None)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.13 on 2018-10-21 11:49
from django.db import migrations, models
import django.db.models.deletion
import django.db.models.manager


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='TreeItem',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('path', models.CharField(blank=True, default='', max_length=255)),
                ('part_of', models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='has_treeitem_children', to='nano_tools_tests.TreeItem')),
            ],
            options={
                'abstract': False,
            },
            managers=[
                ('tree', django.db.models.manager.Manager()),
            ],
        ),
    ]



# coding: utf-8

from django.utils.timezone import now as tznow
from django.test import TestCase

from nano.tools.models import *
from nano.tools import *
from nano.tools.tests.models import TreeItem


class TreeMixinTest(TestCase):

    def setUp(self):
        self.root1 = TreeItem()
        self.root1.save() 
        self.child1 = TreeItem(part_of=self.root1)
        self.child1.save()
        self.child2 = TreeItem(part_of=self.root1)
        self.child2.save()
        self.grandchild1 = TreeItem(part_of=self.child2)
        self.grandchild1.save()

    def test_roots(self):
        items = set(self.child1.roots())
        self.assertEqual(items, set((self.root1,)))

    def test_get_path(self):
        items = self.child1.get_path()
        self.assertEqual(items, [self.root1,self.child1])

    def test_descendants(self):
        items = set(self.root1.descendants())
        self.assertEqual(items, set([self.child1, self.child2, self.grandchild1]))
        items = list(self.child1.descendants())
        self.assertEqual(items, [])

    def test_children(self):
        items = set(self.root1.children())
        self.assertEqual(items, set([self.child1, self.child2]))
        items = list(self.child1.children())
        self.assertEqual(items, [])

    def test_parent(self):
        items = self.root1.parent()
        self.assertEqual(items, None)
        items = self.child1.parent()
        self.assertEqual(items, self.root1)

    def test_siblings(self):
        items = list(self.root1.siblings())
        self.assertEqual(items, [])
        items = set(self.child1.siblings())
        self.assertEqual(items, set([self.child1, self.child2]))

    def test_is_sibling_of(self):
        items = self.root1.is_sibling_of(self.child1)
        self.assertFalse(items)
        items = self.child1.is_sibling_of(self.child2)
        self.assertTrue(items)

    def test_is_child_of(self):
        items = self.root1.is_child_of(self.child1)
        self.assertFalse(items)
        items = self.child1.is_child_of(self.root1)
        self.assertTrue(items)

    def test_is_root(self):
        items = self.root1.is_root()
        self.assertTrue(items)
        items = self.child1.is_root()
        self.assertFalse(items)

    def test_is_leaf(self):
        items = self.root1.is_leaf()
        self.assertFalse(items)
        items = self.child1.is_leaf()
        self.assertTrue(items)

class TextTest(TestCase):
    pass

class FunctionTest(TestCase):

    def test_nullfunction(self):
        result = nullfunction(1, 2, three=3)
        self.assertEqual(result, 1)
        result = nullfunction(1, 2)
        self.assertEqual(result, 1)
        result = nullfunction(1, three=3)
        self.assertEqual(result, 1)
        result = nullfunction(1)
        self.assertEqual(result, 1)

    def test_pop_error(self):

        class Foo(object):

            session = {'error': 'baa'}

        request = Foo()
        result = pop_error(request)
        self.assertEqual(result, 'baa')
        self.assertFalse('error' in request.session)

        request.session = {}
        result = pop_error(request)
        self.assertEqual(result, None)

    def test_asciify(self):
        self.assertEqual('', asciify(''))
        self.assertEqual('abcABC123', asciify('abcABC123'))
        self.assertEqual('blabrsyltety', asciify('blåbærsyltetøy'))

    def test_grouper_nothing_to_group(self):
        self.assertEqual([], list(grouper(0, '')))
        self.assertEqual([], list(grouper(0, 'abc')))
        self.assertEqual([], list(grouper(1, '')))
        self.assertEqual([], list(grouper(2, '')))

    def test_grouper(self):
        self.assertEqual(
            [('a',), ('b',), ('c',)],
            list(grouper(1, 'abc'))
        )
        self.assertEqual(
            [('a', u'b'), ('c', None)],
            list(grouper(2, 'abc'))
        )
        self.assertEqual(
            [(u'a', u'b', u'c')],
            list(grouper(3, 'abc'))
        )
        self.assertEqual(
            [(u'a', u'b', u'c', None)],
            list(grouper(4, 'abc'))
        )

    def test_grouper_fillvalue(self):
        self.assertEqual(
            [('a',), ('b',), ('c',)],
            list(grouper(1, 'abc', fillvalue='X'))
        )
        self.assertEqual(
            [('a', 'b'), ('c', 'X')],
            list(grouper(2, 'abc', fillvalue='X'))
        )
        self.assertEqual(
            [('a', 'b', 'c')],
            list(grouper(3, 'abc', fillvalue='X'))
        )
        self.assertEqual(
            [('a', 'b', 'c', 'X')],
            list(grouper(4, 'abc', fillvalue='X'))
        )



"""
Wrapper for loading templates from the filesystem.
"""

from django.apps import apps
from django.conf import settings
from django.template import TemplateDoesNotExist
from django.template.base import Origin
from django.utils._os import safe_join

from django.template.loaders.base import Loader


class ChunkLoader(Loader):
    is_usable = True

    def _get_contents(self, template_name):
        chunk_model = apps.get_model('chunk', 'Chunk')
        try:
            chunk = chunk_model.objects.get(slug=template_name)
            return chunk.content
        except chunk_model.DoesNotExist:
            error_msg = "Couldn't find a chunk named %s" % template_name
            raise TemplateDoesNotExist(error_msg)

    # Django 1.11
    def load_template_source(self, template_name, template_dirs=None):
        return self._get_contents(template_name), template_name

    # Django 2.0+
    def get_contents(self, origin):
        return self._get_contents(origin.name)

    def get_template_sources(self, template_name):
        return [Origin(template_name, template_name, loader=self)]



from django.urls import path

from nano.badge import views


urlpatterns = [
    path('<int:pk>/',  views.show_badge, name='badge-detail'),
    path('',           views.list_badges, name='badge-list'),
]



from functools import partial

from django.contrib import admin

from nano.blog.models import Entry
from nano.blog.settings import NANO_BLOG_TAGS


if NANO_BLOG_TAGS:
    from taggit.models import TaggedItem

    MAGICAL_TAGS = ('devel', 'pinned', 'news')


    class BlogHasTagFilter(admin.SimpleListFilter):
        title = 'having tags'
        parameter_name = 'has_any_tags'

        def lookups(self, request, model_admin):
            return (
                ('yes', 'Yes'),
                ('no', 'No'),
            )

        def queryset(self, request, queryset):
            if self.value() == 'yes':
                return queryset.exclude(tags=None)
            elif self.value() == 'no':
                return queryset.filter(tags=None)
            return queryset

    class BlogTagFilter(admin.SimpleListFilter):
        title = 'specific tags'
        parameter_name = 'has_tag'

        def _choices(self):
            tagged_items = TaggedItem.objects.filter(content_type__model=Entry._meta.model_name)
            blog_tags = tuple(tagged_items.values_list("tag__slug", "tag__name").distinct())
            return blog_tags

        def lookups(self, request, model_admin):
            return self._choices()

        def queryset(self, request, queryset):
            tags = [slug for (slug, _) in self._choices()]
            value = self.value()
            if value in tags:
                return queryset.filter(tags__slug=value)
            return queryset

    def _tag(modeladmin, request, queryset, tag):
        for entry in queryset:
            entry.tags.add(tag)

    def _untag(modeladmin, request, queryset, tag):
        for entry in queryset:
            entry.tags.remove(tag)

    def tagaction_factory():
        actions = []
        for tag in MAGICAL_TAGS:
            action = partial(_tag, tag=tag)
            action.short_description = f"Tag selected entries with '{tag}'"
            action.__name__ = f"tag_{tag}"
            actions.append(action)
            action = partial(_untag, tag=tag)
            action.short_description = f"Remove '{tag}'-tag from selected entries"
            action.__name__ = f"untag_{tag}"
            actions.append(action)
        return actions


@admin.register(Entry)
class EntryAdmin(admin.ModelAdmin):
    model = Entry
    list_display = ('headline', 'pub_date')
    search_fields = ('headline', 'pub_date')
    date_hierarchy = 'pub_date'

    if NANO_BLOG_TAGS:
        list_display = list_display + ("tag_list",)
        list_filter = [BlogHasTagFilter, BlogTagFilter]
        actions = tagaction_factory()

        def get_queryset(self, request):
            return super().get_queryset(request).prefetch_related('tags')

        def tag_list(self, obj):
            return ", ".join(o.name for o in obj.tags.all())



from django.test import TestCase

from nano.faq.models import QA
from nano.faq.views import ListFAQView

class QATest(TestCase):

    def test_str(self):
        item = QA(question='blbl', answer='fofo')
        self.assertEqual(str(item), item.question)

    def test_save(self):
        item = QA(question='blbl', answer='fofo')
        item.save()
        self.assertNotEqual(item.last_modified, None)

class ListFAQViewTest(TestCase):

    def test_get_context_data(self):
        view = ListFAQView()
        view.object_list = None
        context = view.get_context_data(object_list=None)
        self.assertEqual(context['me'], 'faq')



from django.test import TestCase
from django.contrib.auth import get_user_model

from nano.badge.models import Badge
from nano.badge import add_badge, batchbadge
from nano.badge.views import ListBadgeView


class BadgeTest(TestCase):

    def test_str(self):
        item = Badge(name='test', description='Test')
        self.assertEqual(str(item), item.name)


class BadgeManagerTest(TestCase):

    def setUp(self):
        self.User = get_user_model()
        self.badge = Badge.objects.create(name='test', description='Test')
        self.users = []
        for i in range(10):
            self.users.append(self.User.objects.create(username='user%i' % i))
        for user in self.users[:5]:
            self.badge.receivers.add(user)

    def test_get_all_recipients(self):
        result = list(Badge.objects.get_all_recipients())
        self.assertEqual(result, self.users[:5])

    def test_get_all_nonrecipients(self):
        result = list(Badge.objects.get_all_nonrecipients())
        self.assertEqual(result, self.users[5:])


class BadgeFunctionsTest(TestCase):

    def setUp(self):
        self.User = get_user_model()
        self.badge = Badge.objects.create(name='test', description='Test')
        self.users = []
        for i in range(10):
            self.users.append(self.User.objects.create(username='user%i' % i))
        for user in self.users[:2]:
            self.badge.receivers.add(user)

    def test_add_badge(self):
        add_badge(self.badge, self.users[2])
        self.assertEqual(list(self.users[2].badges.all()), [self.badge])
        add_badge(self.badge, self.users[0])
        self.assertEqual(list(self.users[0].badges.all()), [self.badge])

    def test_batchbadge(self):
        self.User = get_user_model()
        self.assertEqual(list(self.badge.receivers.all()), self.users[:2])
        batchbadge(self.badge, self.User.objects.all())
        self.assertEqual(list(self.badge.receivers.all()), self.users)


class BadgeMixinTest(TestCase):

    def test_get_context_data(self):
        bm = ListBadgeView()
        bm.object_list = None  # set during as_view in 1.6
        context = bm.get_context_data(object_list=None)  # argument in 1.5
        self.assertEqual(context['me'], 'badge')



from django.shortcuts import render
from django.urls import include, path

from nano.user import views


signup_done_data = {'template_name': 'nano/user/signup_done.html'}

# 'project_name' should be a setting
password_reset_data = {'project_name': 'CALS'}

urlpatterns = [
    path('password/', include([
        path('reset/',   views.password_reset, password_reset_data, name='nano_user_password_reset'),
        path('change/',  views.password_change, name='nano_user_password_change'),
    ])),
    path('signup/', include([
        path('done/',      render, signup_done_data, name='nano_user_signup_done'),
        path('',           views.signup, name='nano_user_signup'),
    ])),
]
